@@ -29,6 +29,7 @@ from io.output import *
 from gen.container import *
 from gen.structure import *
 from gen.layout import *
+from gen.image import *
 from ref.label import *
 from post.postprocess import *
 
@@ -39,24 +40,60 @@ class Float(Container):
   def __init__(self):
     self.parser = InsetParser()
     self.output = TaggedOutput().settag('div class="float"', True)
-    self.parent = None
+    self.parentfloat = None
     self.children = []
     self.number = None
 
   def process(self):
-    "Get the float type"
+    "Get the float type."
     self.type = self.header[2]
-    self.embed('div class="' + self.type + '"')
-    for float in self.searchall(Float):
+    self.embeddedtag = 'div class="' + self.type + '"'
+    self.processinsides()
+    self.embed(self.embeddedtag)
+
+  def processinsides(self):
+    "Process all floats and images inside."
+    floats = self.searchall(Float)
+    for float in floats:
       float.output.tag = float.output.tag.replace('div', 'span')
-      float.parent = self
+      float.parentfloat = self
       self.children.append(float)
+    if len(floats) > 0:
+      return
+    images = self.searchall(Image)
+    if len(images) != 1:
+      return
+    image = images[0]
+    if not image.width:
+      return
+    if not '%' in image.width:
+      return
+    image.type = 'figure'
+    self.embeddedtag += ' style="width: ' + image.width + ';"'
+    image.width = None
 
   def embed(self, tag):
     "Embed the whole contents in a div"
     tagged = TaggedText().complete(self.contents, tag, True)
     self.contents = [tagged]
 
+  def searchinside(self, contents, type):
+    "Search for a given type in the contents"
+    list = []
+    for element in contents:
+      list += self.searchinelement(element, type)
+    return list
+
+  def searchinelement(self, element, type):
+    "Search for a given type outside floats"
+    if isinstance(element, Float):
+      return []
+    if isinstance(element, type):
+      return [element]
+    if not isinstance(element, Container):
+      return []
+    return self.searchinside(element.contents, type)
+
   def __unicode__(self):
     "Return a printable representation"
     return 'Floating inset of type ' + self.type
@@ -142,29 +179,12 @@ class PostFloat(object):
 
   def postprocess(self, float, last):
     "Move the label to the top and number the caption"
-    captions = self.searchcaptions(float.contents)
+    captions = float.searchinside(float.contents, Caption)
     for caption in captions:
       self.postlabels(float, caption)
       self.postnumber(caption, float)
     return float
 
-  def searchcaptions(self, contents):
-    "Search for captions in the contents"
-    list = []
-    for element in contents:
-      list += self.searchcaptionelement(element)
-    return list
-
-  def searchcaptionelement(self, element):
-    "Search for captions outside floats"
-    if isinstance(element, Float):
-      return []
-    if isinstance(element, Caption):
-      return [element]
-    if not isinstance(element, Container):
-      return []
-    return self.searchcaptions(element.contents)
-
   def postlabels(self, float, caption):
     "Search for labels and move them to the top"
     labels = caption.searchremove(Label)
@@ -175,19 +195,20 @@ class PostFloat(object):
   def postnumber(self, caption, float):
     "Number the caption"
     self.numberfloat(float)
-    prefix = TranslationConfig.floats[float.type]
-    caption.contents.insert(0, Constant(prefix + float.number + u' '))
+    caption.contents.insert(0, Constant(float.entry + u' '))
 
   def numberfloat(self, float):
     "Number a float if it isn't numbered"
     if float.number:
       return
-    if float.parent:
-      self.numberfloat(float.parent)
-      index = float.parent.children.index(float)
-      float.number = float.parent.number + NumberGenerator.letters[index + 1]
+    if float.parentfloat:
+      self.numberfloat(float.parentfloat)
+      index = float.parentfloat.children.index(float)
+      float.number = NumberGenerator.letters[index + 1].lower()
+      float.entry = '(' + float.number + ')'
     else:
       float.number = NumberGenerator.instance.generatechaptered(float.type)
+      float.entry = TranslationConfig.floats[float.type] + float.number
 
 class PostWrap(PostFloat):
   "For a wrap: exactly like a float"
