@@ -22,8 +22,9 @@
 # Alex 20090308
 # eLyXer image treatment
 
-import os
 import struct
+import sys
+import os
 from util.trace import Trace
 from gen.container import *
 from io.path import *
@@ -32,12 +33,14 @@ from io.path import *
 class Image(Container):
   "An embedded image"
 
-  converter = True
   ignoredtexts = ImageConfig.size['ignoredtexts']
 
   def __init__(self):
     self.parser = InsetParser()
     self.output = ImageOutput()
+    self.type = 'embedded'
+    self.width = None
+    self.height = None
 
   def process(self):
     "Place the url, convert the image if necessary."
@@ -46,7 +49,7 @@ class Image(Container):
       Trace.error('Image ' + unicode(self.origin) + ' not found')
       return
     self.destination = self.getdestination(self.origin)
-    self.convert(self.getparams())
+    ImageConverter.instance.convert(self)
     self.setsize()
 
   def getdestination(self, origin):
@@ -63,53 +66,11 @@ class Image(Container):
     destination.removebackdirs()
     return destination
 
-  def convert(self, params):
-    "Convert an image to PNG"
-    if not Image.converter:
-      return
-    if self.origin == self.destination:
-      return
-    if self.destination.exists():
-      if self.origin.getmtime() <= self.destination.getmtime():
-        # file has not changed; do not convert
-        return
-    self.destination.createdirs()
-    command = 'convert '
-    for param in params:
-      command += '-' + param + ' ' + unicode(params[param]) + ' '
-    command += '"' + unicode(self.origin) + '" "'
-    command += unicode(self.destination) + '"'
-    try:
-      result = os.system(command)
-      Trace.debug('ImageMagick Command: "' + command + '"')
-      if result != 0:
-        Trace.error('ImageMagick not installed; images will not be processed')
-        Image.converter = False
-        return
-      Trace.message('Converted ' + unicode(self.origin) + ' to ' +
-          unicode(self.destination))
-    except OSError:
-      Trace.error('Error while converting image ' + self.origin)
-
-  def getparams(self):
-    "Get the parameters for ImageMagick conversion"
-    params = dict()
-    scale = 100
-    if 'scale' in self.parameters:
-      scale = int(self.parameters['scale'])
-    if self.origin.hasext('.svg'):
-      params['density'] = scale
-    elif self.origin.hasext('.jpg') or self.origin.hasext('.png'):
-      params['resize'] = unicode(scale) + '%'
-    elif self.origin.hasext('.pdf'):
-      params['define'] = 'pdf:use-cropbox=true'
-    return params
-
   def setsize(self):
     "Set the size attributes width and height."
-    if self.setifparam('width'):
-      return
-    if self.setifparam('height'):
+    self.setifparam('width')
+    self.setifparam('height')
+    if self.width or self.height:
       return
     imagefile = ImageFile(self.destination)
     width, height = imagefile.getdimensions()
@@ -121,13 +82,64 @@ class Image(Container):
   def setifparam(self, name):
     "Set the value in the container if it exists as a param."
     if not name in self.parameters:
-      return False
+      return
     value = unicode(self.parameters[name])
     for ignored in Image.ignoredtexts:
       if ignored in value:
         value = value.replace(ignored, '')
     setattr(self, name, value)
-    return True
+
+class ImageConverter(object):
+  "A converter from one image file to another."
+
+  active = True
+  instance = None
+
+  def convert(self, image):
+    "Convert an image to PNG"
+    if not ImageConverter.active:
+      return
+    if image.origin == image.destination:
+      return
+    if image.destination.exists():
+      if image.origin.getmtime() <= image.destination.getmtime():
+        # file has not changed; do not convert
+        return
+    image.destination.createdirs()
+    command = 'convert '
+    params = self.getparams(image)
+    for param in params:
+      command += '-' + param + ' ' + unicode(params[param]) + ' '
+    command += '"' + unicode(image.origin) + '" "'
+    command += unicode(image.destination) + '"'
+    try:
+      Trace.debug('ImageMagick Command: "' + command + '"')
+      result = os.system(command.encode(sys.getfilesystemencoding()))
+      if result != 0:
+        Trace.error('ImageMagick not installed; images will not be processed')
+        ImageConverter.active = False
+        return
+      Trace.message('Converted ' + unicode(image.origin) + ' to ' +
+          unicode(image.destination))
+    except OSError, exception:
+      Trace.error('Error while converting image ' + unicode(image.origin)
+          + ': ' + unicode(exception))
+
+  def getparams(self, image):
+    "Get the parameters for ImageMagick conversion"
+    params = dict()
+    scale = 100
+    if 'scale' in image.parameters:
+      scale = int(image.parameters['scale'])
+    if image.origin.hasext('.svg'):
+      params['density'] = scale
+    elif image.origin.hasext('.jpg') or image.origin.hasext('.png'):
+      params['resize'] = unicode(scale) + '%'
+    elif image.origin.hasext('.pdf'):
+      params['define'] = 'pdf:use-cropbox=true'
+    return params
+
+ImageConverter.instance = ImageConverter()
 
 class ImageFile(object):
   "A file corresponding to an image (JPG or PNG)"
@@ -217,14 +229,13 @@ class ImageOutput(object):
 
   def gethtml(self, container):
     "Get the HTML output of the image as a list"
-    cssclass = 'embedded'
-    html = ['<img class="' + cssclass + '"']
+    html = ['<img class="' + container.type + '"']
     if container.origin.exists():
       html.append(' src="' + container.destination.url +
           '" alt="' + ImageOutput.figure + ' ' + container.destination.url + '"')
-      if hasattr(container, 'width'):
+      if container.width:
         html.append(' width="' + container.width + '"')
-      if hasattr(container, 'height'):
+      if container.height:
         html.append(' height="' + container.height + '"')
     else:
       html.append(' src="' + container.origin.url + '"')
