@@ -63,6 +63,7 @@ class Title(Layout):
     self.type = 'title'
     self.output.tag = 'h1 class="title"'
     self.title = self.extracttext()
+    TitleOutput.title = self.title
     Trace.message('Title: ' + self.title)
 
 class Author(Layout):
@@ -163,6 +164,7 @@ class PlainLayout(Layout):
   "A plain layout"
 
   def process(self):
-    "Do nothing"
+    "Output just as contents."
+    self.output = ContentsOutput()
     self.type = 'Plain'
 
