@@ -38,9 +38,14 @@ class LyxHeader(Container):
 
   def process(self):
     "Find pdf title"
-    key = ContainerConfig.header['pdftitle']
+    key = HeaderConfig.parameters['pdftitle']
     if key in self.parameters:
       TitleOutput.pdftitle = self.parameters[key]
+    key = HeaderConfig.parameters['documentclass']
+    if key in self.parameters:
+      documentclass = self.parameters[key]
+      if documentclass in HeaderConfig.styles['article']:
+        NumberGenerator.startinglevel = 1
 
 class LyxFooter(Container):
   "Reads the footer, outputs the HTML footer"
