@@ -31,6 +31,7 @@ from gen.factory import *
 from gen.toc import *
 from gen.inset import *
 from gen.basket import *
+from gen.integral import *
 from post.postprocess import *
 from post.postlist import *
 from post.posttable import *
@@ -40,28 +41,35 @@ from math.postformula import *
 class eLyXerConverter(object):
   "Converter for a document in a lyx file. Places all output in a given basket."
 
-  latestbasket = None
+  currentbasket = None
+
+  def __init__(self):
+    self.filtering = False
 
   def setio(self, ioparser):
     "Set the InOutParser"
-    self.reader = LineReader(ioparser.filein)
-    self.writer = LineWriter(ioparser.fileout)
-    if Options.toc:
-      self.basket = TOCBasket()
-    else:
-      if not Options.splitpart:
-        self.basket = WriterBasket()
-      else:
-        self.basket = SplittingBasket()
-    self.basket.setwriter(self.writer)
-    eLyXerConverter.latestbasket = self.basket
+    self.reader = ioparser.getreader()
+    self.basket = self.getbasket()
+    eLyXerConverter.currentbasket = self.basket
+    self.basket.setwriter(ioparser.getwriter())
     return self
 
+  def getbasket(self):
+    "Get the appropriate basket for the current options."
+    if Options.toc:
+      return TOCBasket()
+    if Options.splitpart:
+      return SplittingBasket()
+    if Options.memory:
+      return MemoryBasket()
+    return WriterBasket()
+
   def embed(self, reader):
     "Embed the results for a new input file into the latest output file."
     "Header and footer are ignored. Useful for embedding one document inside another."
+    self.filtering = True
     self.reader = reader
-    self.basket = eLyXerConverter.latestbasket.getfilterheader()
+    self.basket = eLyXerConverter.currentbasket
     return self
 
   def convert(self):
@@ -77,11 +85,26 @@ class eLyXerConverter(object):
     factory = ContainerFactory()
     self.postproc = Postprocessor()
     while not self.reader.finished():
-      containers = factory.createsome(self.reader)
-      for container in containers:
+      container = factory.createcontainer(self.reader)
+      if container and not self.filtered(container):
         container = self.postproc.postprocess(container)
         self.basket.write(container)
-    self.basket.finish()
+    if not self.filtering:
+      self.basket.finish()
+
+  def filtered(self, container):
+    "Find out if the container is a header or footer and must be filtered."
+    if not self.filtering:
+      return False
+    if container.__class__ in [LyxHeader, LyxFooter]:
+      return True
+    return False
+
+  def __unicode__(self):
+    "Printable representation."
+    string = 'Converter with filtering ' + unicode(self.filtering)
+    string += ' and basket ' + unicode(self.basket)
+    return string
 
 class InOutParser(object):
   "Parse in and out arguments"
@@ -112,6 +135,14 @@ class InOutParser(object):
       raise Exception('Unused arguments: ' + unicode(args))
     return self
 
+  def getreader(self):
+    "Get the resulting reader."
+    return LineReader(self.filein)
+
+  def getwriter(self):
+    "Get the resulting writer."
+    return LineWriter(self.fileout)
+
   def readdir(self, filename, diroption):
     "Read the current directory if needed"
     if getattr(Options, diroption) != None:
