@@ -49,29 +49,26 @@ class PostLayout(object):
   "Numerate an indexed layout"
 
   processedclass = Layout
-  unique = NumberingConfig.layouts['unique']
-  ordered = NumberingConfig.layouts['ordered']
-
-  def __init__(self):
-    self.generator = NumberGenerator.instance
 
   def postprocess(self, layout, last):
     "Generate a number and place it before the text"
     if self.containsappendix(layout):
       self.activateappendix()
-    if layout.type in PostLayout.unique:
-      number = self.generator.generateunique(layout.type)
-      text = TranslationConfig.constants[layout.type] + ' ' + number + u'.'
-    elif layout.type in PostLayout.ordered:
-      level = PostLayout.ordered.index(layout.type)
-      number = self.generator.generate(level)
-      text = number
+    if NumberGenerator.instance.isunique(layout):
+      layout.number = NumberGenerator.instance.generateunique(layout.type)
+      layout.entry = TranslationConfig.constants[layout.type] + ' ' + layout.number
+      text = layout.entry + '.'
+    elif NumberGenerator.instance.isordered(layout):
+      layout.number = NumberGenerator.instance.generateordered(layout.type)
+      text = layout.number
+      layout.entry = TranslationConfig.constants[layout.type] + ' ' + layout.number
+      layout.level = NumberGenerator.instance.getlevel(layout.type)
+      layout.output.tag = layout.output.tag.replace('?', unicode(layout.level))
     else:
       return layout
-    layout.number = number
-    anchor = 'toc-' + layout.type + '-' + number
-    link = Link().complete(text, anchor=anchor, type='toc')
-    layout.contents.insert(0, link)
+    key = 'toc-' + layout.type + '-' + layout.number
+    label = Label().create(text, key, type='toc')
+    layout.contents.insert(0, label)
     layout.contents.insert(1, Constant(u' '))
     return layout
 
@@ -84,7 +81,7 @@ class PostLayout(object):
 
   def activateappendix(self):
     "Change first number to letter, and chapter to appendix"
-    self.generator.number = ['-']
+    NumberGenerator.instance.number = ['-']
 
 class PostStandard(object):
   "Convert any standard spans in root to divs"
