@@ -31,33 +31,51 @@ from util.numbering import *
 from ref.link import *
 
 
-class Label(Container):
+class Label(Link):
   "A label to be referenced"
 
   names = dict()
 
-  def __init__(self):
-    self.parser = InsetParser()
-    self.output = LinkOutput()
-
   def process(self):
-    self.anchor = self.parameters['name']
-    Label.names[self.anchor] = self
-    self.contents = [Constant(' ')]
+    "Process a label container."
+    key = self.parameters['name']
+    self.create(' ', key)
+
+  def create(self, text, key, type = 'Label'):
+    "Create the label for a given key."
+    self.key = key
+    self.complete(text, anchor = key, type = type)
+    Label.names[key] = self
+    if key in Reference.references:
+      for reference in Reference.references[key]:
+        reference.setdestination(self)
+    return self
+
+  def __unicode__(self):
+    "Return a printable representation."
+    return 'Label ' + self.key
 
 class Reference(Link):
   "A reference to a label"
 
-  def __init__(self):
-    self.parser = InsetParser()
-    self.output = LinkOutput()
-    self.direction = u'↓'
+  references = dict()
 
   def process(self):
-    key = self.parameters['reference']
-    self.url = '#' + key
-    if key in Label.names:
-      # already seen
-      self.direction = u'↑'
-    self.contents = [Constant(self.direction)]
+    "Read the reference and set the arrow."
+    self.key = self.parameters['reference']
+    if self.key in Label.names:
+      direction = u'↑'
+      label = Label.names[self.key]
+    else:
+      direction = u'↓'
+      label = Label().complete(' ', self.key, 'preref')
+    self.setdestination(label)
+    self.contents = [Constant(direction)]
+    if not self.key in Reference.references:
+      Reference.references[self.key] = []
+    Reference.references[self.key].append(self)
+
+  def __unicode__(self):
+    "Return a printable representation."
+    return 'Reference ' + self.key
 
