@@ -34,10 +34,19 @@ class Link(Container):
   "A link to another part of the document"
 
   def __init__(self):
-    self.contents = list()
+    Container.__init__(self)
+    self.parser = InsetParser()
     self.output = LinkOutput()
+    self.anchor = None
+    self.url = None
+    self.type = None
+    self.page = None
+    self.target = None
+    if Options.target:
+      self.target = Options.target
 
   def complete(self, text, anchor = None, url = None, type = None):
+    "Complete the link."
     self.contents = [Constant(text)]
     if anchor:
       self.anchor = anchor
@@ -45,45 +54,63 @@ class Link(Container):
       self.url = url
     if type:
       self.type = type
-    if Options.target:
-      self.target = Options.target
     return self
 
-class ListOf(Container):
-  "A list of entities (figures, tables, algorithms)"
+  def setdestination(self, destination):
+    "Set another link as the destination of this one."
+    self.destination = destination
+    if not destination.anchor:
+      Trace.error('Missing anchor in link destination ' + unicode(destination))
+      return
+    self.url = '#' + destination.anchor
+    if destination.page:
+      self.url = destination.page + self.url
+
+  def setmutualdestination(self, destination):
+    "Set another link as destination, and set its destination to this one."
+    self.setdestination(destination)
+    destination.setdestination(self)
+
+class ListInset(Container):
+  "An inset with a list, normally made of links."
 
   def __init__(self):
-    self.parser = BoundedParser()
-    self.output = TaggedOutput().settag('div class="list"', True)
+    self.parser = InsetParser()
+    self.output = ContentsOutput()
+
+  def sortdictionary(self, dictionary):
+    "Sort all entries in the dictionary"
+    keys = dictionary.keys()
+    # sort by name
+    keys.sort()
+    return keys
+
+class ListOf(ListInset):
+  "A list of entities (figures, tables, algorithms)"
 
   def process(self):
     "Parse the header and get the type"
     self.type = self.header[2]
-    self.contents = [Constant(TranslationConfig.lists[self.type])]
+    text = TranslationConfig.lists[self.type]
+    self.contents = [TaggedText().constant(text, 'div class="tocheader"', True)]
 
-class TableOfContents(Container):
+class TableOfContents(ListInset):
   "Table of contents"
 
-  def __init__(self):
-    self.parser = BoundedParser()
-    self.output = TaggedOutput().settag('div class="toc"', True)
-
   def process(self):
     "Parse the header and get the type"
-    self.contents = [Constant(TranslationConfig.constants['toc'])]
+    text = TranslationConfig.constants['toc']
+    self.contents = [TaggedText().constant(text, 'div class="tocheader"', True)]
 
 class IndexEntry(Link):
   "An entry in the alphabetical index"
 
   entries = dict()
+  arrows = dict()
 
   namescapes = {'!':'', '|':', ', '  ':' '}
   keyescapes = {' ':'-', '--':'-', ',':''}
 
-  def __init__(self):
-    self.parser = InsetParser()
-    self.output = LinkOutput()
-
   def process(self):
     "Put entry in index"
     if 'name' in self.parameters:
@@ -91,105 +118,80 @@ class IndexEntry(Link):
     else:
       name = self.extracttext()
     self.name = self.escape(name, IndexEntry.namescapes)
-    self.key = self.escape(self.name, IndexEntry.keyescapes)
-    if not self.key in IndexEntry.entries:
-      # no entry; create
-      IndexEntry.entries[self.key] = list()
-    self.index = len(IndexEntry.entries[self.key])
-    IndexEntry.entries[self.key].append(self)
-    self.anchor = 'entry-' + self.key + '-' + unicode(self.index)
-    self.url = '#index-' + self.key
-    self.contents = [Constant(u'↓')]
+    key = self.escape(self.name, IndexEntry.keyescapes)
+    if not key in IndexEntry.entries:
+      # no entry yet; create
+      entry = Link().complete(name, 'index-' + key, None, 'printindex')
+      entry.name = name
+      IndexEntry.entries[key] = entry
+    if not key in IndexEntry.arrows:
+      # no arrows yet; create list
+      IndexEntry.arrows[key] = []
+    self.index = len(IndexEntry.arrows[key])
+    self.complete(u'↓', 'entry-' + key + '-' + unicode(self.index))
+    self.setdestination(IndexEntry.entries[key])
+    arrow = Link().complete(u'↑', 'index-' + key)
+    arrow.setdestination(self)
+    IndexEntry.arrows[key].append(arrow)
 
-class PrintIndex(Container):
+class PrintIndex(ListInset):
   "Command to print an index"
 
-  def __init__(self):
-    self.parser = BoundedParser()
-    self.output = ContentsOutput()
-
   def process(self):
     "Create the alphabetic index"
     index = TranslationConfig.constants['index']
     self.contents = [TaggedText().constant(index, 'h1 class="index"'),
         Constant('\n')]
-    for key in self.sortentries():
-      name = IndexEntry.entries[key][0].name
-      entry = [Link().complete(name, 'index-' + key, None, 'printindex'),
-          Constant(': ')]
-      contents = [TaggedText().complete(entry, 'i')]
-      contents += self.createarrows(key, IndexEntry.entries[key])
+    for key in self.sortdictionary(IndexEntry.entries):
+      entry = IndexEntry.entries[key]
+      entrytext = [IndexEntry.entries[key], Constant(': ')]
+      contents = [TaggedText().complete(entrytext, 'i')]
+      contents += self.extractarrows(key)
       self.contents.append(TaggedText().complete(contents, 'p class="printindex"',
           True))
 
-  def sortentries(self):
-    "Sort all entries in the index"
-    keys = IndexEntry.entries.keys()
-    # sort by name
-    keys.sort()
-    return keys
-
-  def createarrows(self, key, entries):
-    "Create an entry in the index"
+  def extractarrows(self, key):
+    "Extract all arrows (links to the original reference) for a key."
     arrows = []
-    for entry in entries:
-      link = Link().complete(u'↑', 'index-' + entry.key,
-          '#entry-' + entry.key + '-' + unicode(entry.index))
-      arrows += [link, Constant(u', \n')]
+    for arrow in IndexEntry.arrows[key]:
+      arrows += [arrow, Constant(u', \n')]
     return arrows[:-1]
 
 class NomenclatureEntry(Link):
   "An entry of LyX nomenclature"
 
-  entries = {}
-
-  def __init__(self):
-    self.parser = InsetParser()
-    self.output = LinkOutput()
+  entries = dict()
 
   def process(self):
     "Put entry in index"
-    self.symbol = self.parameters['symbol']
-    self.description = self.parameters['description']
-    self.key = self.symbol.replace(' ', '-').lower()
-    NomenclatureEntry.entries[self.key] = self
-    self.anchor = 'noment-' + self.key
-    self.url = '#nom-' + self.key
-    self.contents = [Constant(u'↓')]
+    symbol = self.parameters['symbol']
+    description = self.parameters['description']
+    key = symbol.replace(' ', '-').lower()
+    if key in NomenclatureEntry.entries:
+      Trace.error('Duplicated nomenclature entry ' + key)
+    self.complete(u'↓', 'noment-' + key)
+    entry = Link().complete(u'↑', 'nom-' + key)
+    entry.symbol = symbol
+    entry.description = description
+    self.setmutualdestination(entry)
+    NomenclatureEntry.entries[key] = entry
 
-class PrintNomenclature(Container):
+class PrintNomenclature(ListInset):
   "Print all nomenclature entries"
 
-  def __init__(self):
-    self.parser = InsetParser()
-    self.output = ContentsOutput()
-
   def process(self):
-    self.keys = self.sortentries()
     nomenclature = TranslationConfig.constants['nomenclature']
-    self.contents = [TaggedText().constant(nomenclature, 'h1 class="nomenclature"')]
-    for key in self.keys:
+    self.contents = [TaggedText().constant(nomenclature,
+      'h1 class="nomenclature"')]
+    for key in self.sortdictionary(NomenclatureEntry.entries):
       entry = NomenclatureEntry.entries[key]
-      contents = [Link().complete(u'↑', 'nom-' + key, '#noment-' + key)]
-      contents.append(Constant(entry.symbol + u' '))
-      contents.append(Constant(entry.description))
+      contents = [entry, Constant(entry.symbol + u' ' + entry.description)]
       text = TaggedText().complete(contents, 'div class="Nomenclated"', True)
       self.contents.append(text)
 
-  def sortentries(self):
-    "Sort all entries in the index"
-    keys = NomenclatureEntry.entries.keys()
-    # sort by name
-    keys.sort()
-    return keys
-
 class URL(Link):
   "A clickable URL"
 
-  def __init__(self):
-    self.parser = InsetParser()
-    self.output = LinkOutput()
-
   def process(self):
     "Read URL from parameters"
     name = self.escape(self.parameters['target'])
@@ -215,14 +217,14 @@ class LinkOutput(object):
   def gethtml(self, container):
     "Get the HTML code for the link"
     type = container.__class__.__name__
-    if hasattr(container, 'type'):
+    if container.type:
       type = container.type
     tag = 'a class="' + type + '"'
-    if hasattr(container, 'anchor'):
+    if container.anchor:
       tag += ' name="' + container.anchor + '"'
-    if hasattr(container, 'url'):
+    if container.url:
       tag += ' href="' + container.url + '"'
-    if hasattr(container, 'target'):
+    if container.target:
       tag += ' target="' + container.target + '"'
     text = TaggedText().complete(container.contents, tag)
     return text.gethtml()
